@@ -19,7 +19,7 @@
  * along with "Otags reloaded". If not, see
  * <http://www.gnu.org/licenses/>.
  * 
- * $Id: otags.ml,v 1.25 2012-01-26 19:56:27 tews Exp $
+ * $Id: otags.ml,v 1.27 2012-03-28 20:46:10 tews Exp $
  * 
  * main module with main function
  * 
@@ -29,6 +29,7 @@ open Conf
 open Global
 open Types
 open Source_channel
+open Monitor_line_directive
 open Translate_location
 open Camlp4_names
 open Parser_factory
@@ -56,13 +57,17 @@ let open_input_file file =
 	prerr_endline msg;
       raise Skip_entry
 
-let camlp4_printer_option = ["-printer"; "d"]
+let otags_dump_ast_module = ["-printer"; "otags_dump_ast.cma"]
 
 let make_camlp4_command unit file =
   let (camlp4, rest_parsers) = camlp4_variant !parser_list in
-  let camlp4 = Filename.concat camlp4_path camlp4 in
+  let camlp4 = Filename.concat camlp4_exec_path camlp4 in
   let dir_options =
-    List.fold_right (fun d res -> "-I" :: d :: res) !camlp4_search_path [] in
+    List.fold_right 
+      (fun d res -> "-I" :: d :: res)
+      (camlp4_mod_path :: !camlp4_search_path)
+      []
+  in
   let parser_options = 
     List.fold_right (fun p res -> "-parser" :: p :: res) rest_parsers [] in
   let unit_option = match unit with
@@ -70,31 +75,31 @@ let make_camlp4_command unit file =
     | Structure -> "-impl"
   in
   let options = 
-    camlp4 :: camlp4_printer_option 
-    @ dir_options 
+    camlp4 :: dir_options 
     @ parser_options 
+    @ otags_dump_ast_module
     @ [unit_option; file]
   in
   String.concat " " options
 
 
-(* See Camlp4/Printers/DumpCamlp4Ast.ml and
- * camlp4/Camlp4Parsers/Camlp4AstLoader.ml 
- *)
+(* See also otags_dump_ast.ml *)
 let read_camlp4_dump ic file unit =
   let magic = match unit with
-    | Signature -> Camlp4_config.camlp4_ast_intf_magic_number
-    | Structure -> Camlp4_config.camlp4_ast_impl_magic_number
-  in
-  let magic_len = String.length magic in
-  let input_magic = String.create magic_len 
+    | Signature -> Conf.otags_camlp4_ast_intf_magic
+    | Structure -> Conf.otags_camlp4_ast_impl_magic
   in
   try
-    really_input ic input_magic 0 magic_len;
+    let input_magic = input_line ic in
     if input_magic = magic then 
-      match unit with
-	| Signature -> Sig_ast(input_value ic : sig_item_t)
-	| Structure -> Struct_ast(input_value ic : str_item_t)
+      let line_directives = 
+	(input_value ic : Monitor_line_directive.line_directive_record list) in
+      let comp_unit =
+	match unit with
+	  | Signature -> Sig_ast(input_value ic : sig_item_t)
+	  | Structure -> Struct_ast(input_value ic : str_item_t)
+      in
+      (line_directives, comp_unit)
     else 
       raise Skip_entry
   with
@@ -149,9 +154,9 @@ let parse_file_externally unit file =
     prerr_endline camlp4;
   end;
   let ast_ic = U.open_process_in camlp4 in
-  let comp_unit = read_camlp4_dump ast_ic file unit in
+  let directives_and_unit = read_camlp4_dump ast_ic file unit in
   match U.close_process_in ast_ic with
-    | U.WEXITED 0 -> comp_unit
+    | U.WEXITED 0 -> directives_and_unit
     | _ -> 
       if not !silent then
 	Printf.eprintf "External camlp4 on %s failed\n" file;
@@ -161,7 +166,10 @@ let parse_file_externally unit file =
 let process_file tagfun unit file =
   let comp_unit = match !use_internal_parsers with
     | true -> parse_file_internally unit file
-    | false -> parse_file_externally unit file
+    | false -> 
+      let (line_directives, comp_unit) = parse_file_externally unit file in
+      parsed_line_directives := line_directives;
+      comp_unit
   in
   prepare_line_directives ();
   tagfun.start_unit file;
