@@ -19,7 +19,7 @@
  * along with "Otags reloaded". If not, see
  * <http://www.gnu.org/licenses/>.
  * 
- * $Id: tags.ml,v 1.19 2012-01-29 15:21:50 tews Exp $
+ * $Id: tags.ml,v 1.24 2012-05-16 20:00:24 tews Exp $
  * 
  * recursive tagging function
  * 
@@ -108,8 +108,9 @@ let rec tag_type write_tag typ = match typ with
       (tag_record_label_decl write_tag)
       (Ast.list_of_ctyp t [])
 
-    (* open polymorphic variant type *)
-  | <:ctyp< [> $_$ ] >>
+  | <:ctyp< $tup:_$ >>			(* tuple *)
+  | <:ctyp< [= $_$ ] >>			(* exact variant type *)
+  | <:ctyp< [> $_$ ] >>		        (* open polymorphic variant type *)
     (* closed polymorphic variant type with no known tags *)
   | <:ctyp< [< $_$ ] >>
     (* closed polymorphic variant type with some known tags *)
@@ -117,16 +118,37 @@ let rec tag_type write_tag typ = match typ with
     (* conjunctive type in polymorphic variants *)
   | <:ctyp< $_$ & $_$ >>
     (* [< A of & int] : impossible type in polymorphic variant types *)
-  | <:ctyp< $_$ of & $_$ >> 
-      -> ()
+  | <:ctyp< $_$ of & $_$ >>
 
+  | <:ctyp< (module $_$) >>		(* module package type *)
+  | <:ctyp< ' $_$ >>			(* type parameter *)
+  | <:ctyp< < $_$ $..:_$ > >>		(* object type *)
+  | <:ctyp< $id:_$ >>			(* type id *)
+  | <:ctyp< # $_$ >>			(* class type *)
+  | <:ctyp< $_$ -> $_$ >>
+  | <:ctyp< $_$ $_$ >>			(* type constructor application *)
+  | <:ctyp< $_$ as $_$ >>
+  | <:ctyp< >>
+    -> ()
+
+  | <:ctyp< ! $_$ . $_$ >>              (* polymorphic type expr *)
+  | <:ctyp< ? $_$ : $_$ >>		(* optional labeled function arg *)
+  | <:ctyp< ~ $_$ : $_$ >>		(* optional labeled function arg *)
+  | <:ctyp< $_$ * $_$ >>		(* list inside tuples *)
+  | <:ctyp< $_$ | $_$ >>		(* constructor list *)
+  | <:ctyp< $_$ of $_$ >>		(* constructor decl *)
+  | <:ctyp< $_$ and $_$ >>		(* variant constructor argument list *)
+  | <:ctyp< mutable $_$ >>		(* mutable decl in records *)
+  | <:ctyp< $_$ , $_$ >>		(* class param list *)
+  | <:ctyp< $_$ ; $_$ >>		(* field list in records or obj types *)
+  | <:ctyp< $_$ : $_$ >>		(* record/obj field decl *)
+  | <:ctyp< ` $_$ >>			(* poly variant *)
+  | <:ctyp< + ' $_$ >>			(* type parameter *)
+  | <:ctyp< - ' $_$ >>			(* type parameter *)
+  | <:ctyp< _ >>			(* anonymous type variable *)
   | <:ctyp< $anti:_$ >>
       -> assert false
 
-  (* XXX *)
-  | _ -> () 
-
-
 
 let tag_class_type_decl write_tag = function
   | <:class_type< $virtual:vf$ $id:<:ident< $lid:lid$ >> as id$ [ $_$] >> ->
@@ -242,10 +264,16 @@ let rec tag_sig_item write_tag sig_item = match sig_item with
     tag_constructor_decl write_tag exid
   | <:sig_item< external $id$ : $_$ = $_$ >> ->
     (* missing location info for the identifier: See #5147.
-     * we need the second word 
+     * We need the second word for normal identifiers and the 
+     * stuff inside the parens for infix identifiers.
      *)
     let wrong_id_loc = translate_loc (Ast.loc_of_sig_item sig_item) in
-    write_tag (Reparse.loc_of_second_word wrong_id_loc) id
+    let loc_without_ext = Reparse.loc_without_first_word wrong_id_loc in
+    let id_loc = match Reparse.start_char loc_without_ext with     
+      | '(' -> Reparse.loc_inside_parens loc_without_ext
+      | _ -> Reparse.loc_of_first_word loc_without_ext
+    in
+    write_tag id_loc id
   | <:sig_item< module $id$ : $mt$ >> -> 
     (* missing location info for the identifier: See #5147.
      * we need the second word 
@@ -288,9 +316,37 @@ let rec tag_sig_item write_tag sig_item = match sig_item with
     List.iter
       (tag_class_type write_tag)
       (Ast.list_of_class_type cts [])
+  | <:sig_item< module rec $mb$ >> ->
+    List.iter 
+      (tag_module_sig_binding write_tag)
+      (Ast.list_of_module_binding mb [])
 
-  (* XXX *)
-  | _ -> ()
+  | <:sig_item< open $_$ >> 
+  | <:sig_item< include $_$ >>
+    -> ()
+
+  | <:sig_item< # $_$ $_$ >>
+  | <:sig_item< $_$ ; $_$ >>
+  | <:sig_item< >>
+  | <:sig_item< $anti:_$ >>
+    -> assert false
+
+
+and tag_module_sig_binding write_tag mod_binding = match mod_binding with
+  | <:module_binding< $id$ : $mtyp:mtyp$ >> ->
+    (* missing location info for the identifier: See #5147
+     * we need the first word
+     *)
+    let wrong_id_loc = translate_loc (Ast.loc_of_module_binding mod_binding) in
+    write_tag (Reparse.loc_of_first_word wrong_id_loc) id;
+    tag_module_type write_tag mtyp
+
+  | <:module_binding< >>
+  | <:module_binding< $_$ and $_$ >> 	(* MbAnd lists are unpacked already *)
+  					(* no definitions in signatures *)
+  | <:module_binding< $_$ : $mtyp:_$ = $mexp:_$ >>
+  | <:module_binding< $anti:_$ >>
+    -> assert false
 
 
 and tag_module_type write_tag = function
@@ -309,12 +365,12 @@ and tag_module_type write_tag = function
       (Ast.list_of_sig_item sig_items [])
 
   | <:module_type< >>			(* emty module type ast *)
+  | <:module_type< ' $_$ >>		(* ??? MtQuo ??? *)
+  | <:module_type< module type of $_$ >>
     -> ()
 
   | <:module_type< $anti:_$ >>		(* anti quotations *)
     -> assert false
-  (* XXX *)
-  | _ -> ()
 
 
 let tag_class_decl write_tag = function 
@@ -494,10 +550,16 @@ let rec tag_str_item write_tag str_item = match str_item with
     tag_constructor_decl write_tag exid
   | <:str_item< external $id$ : $_$ = $_$ >> ->
     (* missing location info for the identifier: See #5147.
-     * we need the second word 
+     * We need the second word for normal identifiers and the 
+     * stuff inside the parens for infix identifiers.
      *)
     let wrong_id_loc = translate_loc (Ast.loc_of_str_item str_item) in
-    write_tag (Reparse.loc_of_second_word wrong_id_loc) id
+    let loc_without_ext = Reparse.loc_without_first_word wrong_id_loc in
+    let id_loc = match Reparse.start_char loc_without_ext with     
+      | '(' -> Reparse.loc_inside_parens loc_without_ext
+      | _ -> Reparse.loc_of_first_word loc_without_ext
+    in
+    write_tag id_loc id
   | <:str_item< module $uid$ = $me$ >> ->
     (* missing location info for the module id: See #5147.
      * we need the second word
@@ -536,12 +598,16 @@ let rec tag_str_item write_tag str_item = match str_item with
     List.iter
       (tag_class_type write_tag)
       (Ast.list_of_class_type cts [])
+  | <:str_item< module rec $mb$ >> ->
+    List.iter
+      (tag_module_def_binding write_tag)
+      (Ast.list_of_module_binding mb [])
+
 
   | <:str_item< open $_$ >>
-  | <:str_item< module rec $_$ >>
   | <:str_item< include $_$ >>
   | <:str_item< $exp:_$ >>
-  | <:str_item< # $_$ $_$ >>
+  | <:str_item< # $_$ $_$ >>		(* directive *)
     -> ()
 
     (* XXX there is no quotation syntax for the following *)
@@ -552,6 +618,24 @@ let rec tag_str_item write_tag str_item = match str_item with
     -> assert false
 
 
+and tag_module_def_binding write_tag mod_binding = match mod_binding with
+  | <:module_binding< $id$ : $mtyp:mtyp$ = $mexp:mexp$ >> ->
+    (* missing location info for the identifier: See #5147
+     * we need the first word
+     *)
+    let wrong_id_loc = translate_loc (Ast.loc_of_module_binding mod_binding) in
+    write_tag (Reparse.loc_of_first_word wrong_id_loc) id;
+    tag_module_type write_tag mtyp;
+    tag_module_expr write_tag mexp
+
+  | <:module_binding< >>
+  | <:module_binding< $_$ and $_$ >> 	(* MbAnd lists are unpacked already *)
+  					(* no specifications in structures *)
+  | <:module_binding< $_$ : $mtyp:_$ >>
+  | <:module_binding< $anti:_$ >>
+    -> assert false
+
+
 and tag_module_expr write_tag = function 
   | <:module_expr< functor ( $_$ : $_$ ) -> $me$ >> ->
     (* XXX tag arg signature? *)
