@@ -165,6 +165,19 @@ type a80 = A81
 
 type a82 = a80 = A81
 
+let ( ! ) a b = a + b
+
+external ( ! ) : 'a ref -> 'a = "%field0"
+
+module rec A82 : A18A = struct
+  type tx = int
+  let compare = compare
+end
+and A83 : sig 
+  val a84 : int 
+end = struct
+  let a84 = 5
+end
 
 (*** Local Variables: ***)
 (*** compile-command: "make -C .. testtrue" ***)
